<?php

namespace Mon\Test\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Mon\Test\Forms\Test as TestForm;
use Mon\Test\Http\Requests\TestRequest;
use Mon\Test\Interfaces\TestRepositoryInterface;
use Mon\Test\Repositories\Eloquent\Filters\TestResourceFilter;
use Mon\Test\Repositories\Eloquent\Presenters\TestListPresenter;

/**
 * Resource controller class for test.
 */
class TestResourceController extends BaseController
{

    /**
     * Initialize test resource controller.
     *
     *
     * @return null
     */
    public function __construct(TestRepositoryInterface $test)
    {
        parent::__construct();
        $this->form = TestForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('mon.test.modules'), 'test', guard_url('test'));
        $this->repository = $test;
    }

    /**
     * Display a list of test.
     *
     * @return Response
     */
    public function index(TestRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(TestResourceFilter::class)
            ->setPresenter(TestListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('test::test.names'))
            ->view('test::test.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display test.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function show(TestRequest $request, TestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('test::test.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('test::test.show')
            ->output();
    }

    /**
     * Show the form for creating a new test.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(TestRequest $request, TestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('test::test.name'))
            ->view('test::test.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new test.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TestRequest $request, TestRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('test::test.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('test/test/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/test/test'))
                ->redirect();
        }

    }

    /**
     * Show test for editing.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function edit(TestRequest $request, TestRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('test::test.name'))
            ->view('test::test.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the test.
     *
     * @param Request $request
     * @param Model   $test
     *
     * @return Response
     */
    public function update(TestRequest $request, TestRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('test::test.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('test/test/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('test/test/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the test.
     *
     * @param Model   $test
     *
     * @return Response
     */
    public function destroy(TestRequest $request, TestRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('test::test.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('test/test/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('test/test/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
